package BenesNetwork;

import java.lang.reflect.Method;
import java.util.Vector;

/**
 * Main class for Benes Network
 * @version 1.0
 */
public class BenesNetwork
{
	private Switch[][] bn;

	private int gyS;
	private int gxS;
    private int nit;
	private int iType;

    /**
     * Benes network initialization
     * @param ynSwitches    - number of switches on y-axis
     * @param xnSwitches    - number of switches on x-axis
     */
    public BenesNetwork( int ynSwitches, int xnSwitches )
    {
		gxS = xnSwitches;
		gyS = ynSwitches;

		bn = new Switch[ ynSwitches ][ xnSwitches ];
		for( int i = 0; i < ynSwitches; i++ )
			for( int j = 0; j < xnSwitches; j++ )
				bn[i][j] = new Switch();
        nit = -1;
    }

    /**
     * Executes Benes Network algorithm for sub network
     *
     * @param input     - array of input values for sub network
     * @param output    - array of output values for sub network
     * @param xnS       - x-coordinate of the switch in the Network
     * @param ynS       - y-coordinate of the switch in the Network
     * @param xlS       - number of switches in subnetwork for x-axis
     * @param ylS       - number of switches in subnetwork for y-axis
     * @param it        - number of iterations to do
     * @param iType     - if 0 then standard, if 1 then loop iterations
     */
    public void execute( int[] input, int[] output,
		int xnS, int ynS, int xlS, int ylS, int it, int iType )
    {
		this.iType = iType;
        if( iType == 1 )
        {
            execute_loop( input, output, xnS, ynS, xlS, ylS, it );
            return;
        }

        // Sets parameters for iterations
        if( it > 0 )    { nit = it; it = 0; }
        if( iType == 0 )
            if( nit != -1 )
            {
                if( nit == 0 ) return;
                nit--;
            }

		// Recursion condition
		if( xlS == 1 )
		{
	        for( int i = 0; i < ylS; i++ )
			{
				bn[ynS+i][xnS].setVisited();
				bn[ynS+i][xnS].setI1( input[i*2] );
				bn[ynS+i][xnS].setI2( input[i*2+1] );
				if( input[i*2] != output[i*2] )
					bn[ynS+i][xnS].setExchange();
			}
			return;
		}

        // Initialization
		int[] i1 = new int[ ylS ];
		int[] i2 = new int[ ylS ];
		int[] o1 = new int[ ylS ];
		int[] o2 = new int[ ylS ];
		int nS;

        for( int i = 0; i < ylS; i++ )
        {
            i1[i] = i2[i] = o1[i] = o2[i] = -1;
        }

        // Processing through the input switches
        for( int i = 0; i < ylS; i++ )
		{
			if( bn[ynS+i][xnS].isVisited() ) continue;

			nS = i;

			do
			{
                // End of the loop iteration condition
                if( iType == 1 )
                    if( nit != -1 )
                    {
                        setSwitches( i1, o1, xnS+1, ynS, xlS-2, ylS/2 );
                        setSwitches( i2, o2, xnS+1, ynS+ylS/2, xlS-2, ylS/2 );
                        if( nit == 0 ) return;
                        nit--;
                    }
                //--

                // Assign values to input and output arrays
                if( bn[ynS+nS][xnS].isVisited() )
                {
    				bn[ynS+nS][xnS].setI2( input[nS*2+1] );
                    if(bn[ynS+nS][xnS].isExchanged())
        				i1[nS] = bn[ynS+nS][xnS].getI2();
                    else
        				i2[nS] = bn[ynS+nS][xnS].getI2();
                }
                else
                {
    				bn[ynS+nS][xnS].setI1( input[nS*2] );
                    if(bn[ynS+nS][xnS].isExchanged())
        				i2[nS] = bn[ynS+nS][xnS].getI1();
                    else
        				i1[nS] = bn[ynS+nS][xnS].getI1();
                }

				bn[ynS+nS][xnS].setVisited();

                // Execute one loop through the algorithm
				nS = loop( input, output, xnS, ynS, xlS, ylS, i1, i2, o1, o2, nS );


				if( nS == -1 )
				{
					break;
				}
                if(!bn[ynS+nS][xnS].isVisited())
                {
                    bn[ynS+nS][xnS].setVisited();
    				bn[ynS+nS][xnS].setI1( input[nS*2] );
                    if(bn[ynS+nS][xnS].isExchanged())
        				i2[nS] = bn[ynS+nS][xnS].getI1();
                    else
        				i1[nS] = bn[ynS+nS][xnS].getI1();
                    continue;
                }
                else    break;
			}
			while( true /*!bn[ynS+nS][xnS].isVisited()*/ );
		}

//        if( iType != 1 )
        {
            setSwitches( i1, o1, xnS+1, ynS, xlS-2, ylS/2 );
            setSwitches( i2, o2, xnS+1, ynS+ylS/2, xlS-2, ylS/2 );
        }

        // Execute algorithm for subnetworks
		execute( i1, o1, xnS+1, ynS, xlS-2, ylS/2, /*it-1*/ 0, iType );
		execute( i2, o2, xnS+1, ynS+ylS/2, xlS-2, ylS/2, /*it-1*/ 0, iType );
    }

    /**
     * Sets switches based on input and output values
     *
     * @param ia        - input values
     * @param oa        - output values
     * @param xnS       - x-coordinate of the switch in the Network
     * @param ynS       - y-coordinate of the switch in the Network
     * @param xlS       - number of switches in subnetwork for x-axis
     * @param ylS       - number of switches in subnetwork for y-axis
     */
    private void setSwitches( int[] ia, int[] oa, int xnS, int ynS, int xlS, int ylS )
    {
        for( int i = ynS; i < ynS + ylS; i++ )
        {
			if( ( iType == 0 || iType == 1 ) && xlS == 1 )
			{
                if( ia[(i-ynS)*2] != oa[(i-ynS)*2] &&
                    oa[(i-ynS)*2] != -1 && ia[(i-ynS)*2] != -1 ||
                    ia[(i-ynS)*2+1] != oa[(i-ynS)*2+1] &&
                    oa[(i-ynS)*2+1] != -1 && ia[(i-ynS)*2+1] != -1 ||
                    ia[(i-ynS)*2] == oa[(i-ynS)*2+1] && oa[(i-ynS)*2+1] != -1 ||
                    ia[(i-ynS)*2+1] == oa[(i-ynS)*2] && oa[(i-ynS)*2] != -1 )
                {
    				bn[i][xnS].setExchange();
                }
	            bn[i][xnS].setI1( ia[(i-ynS)*2] );
        	    bn[i][xnS].setI2( ia[(i-ynS)*2+1] );
			}
			else
			{
   	            bn[i][xnS].setI1( ia[(i-ynS)*2] );
       	        bn[i][xnS+xlS-1].setI1( oa[(i-ynS)*2] );
           	    bn[i][xnS].setI2( ia[(i-ynS)*2+1] );
               	bn[i][xnS+xlS-1].setI2( oa[(i-ynS)*2+1] );
			}
        }
    }

	public Switch[][] getNetwork()
	{
		return bn;
	}

    /**
     * Single loop algorithm
     *
     * @param input     - input values
     * @param output    - output values
     * @param xnS       - x-coordinate of the switch in the Network
     * @param ynS       - y-coordinate of the switch in the Network
     * @param xlS       - number of switches in subnetwork for x-axis
     * @param ylS       - number of switches in subnetwork for y-axis
     * @param i1        - input values for upper sub network
     * @param i2        - input values for lower sub network
     * @param o1        - output values for upper sub network
     * @param o2        - output values for lower sub network
     * @param inS
     * @return  switch number for the following loop iteration
     */
	private int loop(int[] input, int[] output,
		int xnS, int ynS, int xlS, int ylS,
		int[] i1, int[] i2, int[] o1, int[] o2, int inS)
	{
		int sV = -1; // switch value
		int nS = -1; // switch number

		// Loop forward
		int j;
        if( bn[ynS+inS][xnS].isVisited() )
        {
            if( bn[ynS+inS][xnS].isExchanged() )
            {
                // Visited and Exchanged
        		for( j = 0; j < ylS*2; j++ )
	        	{
    	    		if( output[j] == input[inS*2+1] )
	    	    	{
    		    		nS = j/2;
	    		    	o1[nS] = output[j];
		    		    if( j%2 == 0 )
    		    		{
	    		    		sV = output[j+1];
		    		    	bn[ynS+nS][xnS + xlS - 1].setI1(output[j]);
			    		    bn[ynS+nS][xnS + xlS - 1].setI2(output[j+1]);
    				    }
        				else
	        			{
		        			sV = output[j-1];
    			    		bn[ynS+nS][xnS + xlS - 1].setI2(output[j-1]);
	    			    	bn[ynS+nS][xnS + xlS - 1].setI1(output[j]);
		    			    bn[ynS+nS][xnS + xlS - 1].setExchange();
    			    	}
        				bn[ynS+nS][xnS + xlS - 1].setVisited();
	        			break;
                    }
                }
        		if( ylS*2 == j ) return -1;

    	    	// Loop backward
	    	    o2[nS] = sV;
    	    	for( j = 0; j < ylS*2; j++ )
	    	    {
    		    	if( input[j] == sV )
	    		    {
    	    			nS = j/2;
	    	    		i2[nS] = sV;
		    	    	if( j%2 == 0 )
			    	    {
    				    	sV = input[j+1];
	    				    bn[ynS+nS][xnS].setI1(input[j]);
    	    				bn[ynS+nS][xnS].setI2(input[j+1]);
	    	    			bn[ynS+nS][xnS].setExchange();
		    	    	}
    			    	else
	    			    {
    	    				sV = input[j-1];
	    	    			bn[ynS+nS][xnS].setI1(input[j-1]);
		    	    		bn[ynS+nS][xnS].setI2(input[j]);
			    	    }
    				    return nS;
        			}
	        	}
            }
            else
            {
        		for( j = 0; j < ylS*2; j++ )
    	    	{
    	    		if( output[j] == input[inS*2] )
	    	    	{
		    	    	nS = j/2;
			    	    o1[nS] = output[j];
    				    if( j%2 == 0 )
        				{
	        				sV = output[j+1];
		        			bn[ynS+nS][xnS + xlS - 1].setI1(output[j]);
			        		bn[ynS+nS][xnS + xlS - 1].setI2(output[j+1]);
				        }
        				else
	        			{
		        			sV = output[j-1];
    		    			bn[ynS+nS][xnS + xlS - 1].setI2(output[j-1]);
	    		    		bn[ynS+nS][xnS + xlS - 1].setI1(output[j]);
		    		    	bn[ynS+nS][xnS + xlS - 1].setExchange();
    			    	}
        				bn[ynS+nS][xnS + xlS - 1].setVisited();
	        			break;
		        	}
                }

        		if( ylS*2 == j ) return -1;

    	    	// Loop backward
	    	    o2[nS] = sV;
    	    	for( j = 0; j < ylS*2; j++ )
	    	    {
    		    	if( input[j] == sV )
	    		    {
    	    			nS = j/2;
	    	    		i2[nS] = sV;
		    	    	if( j%2 == 0 )
			    	    {
    				    	sV = input[j+1];
	    				    bn[ynS+nS][xnS].setI1(input[j]);
    	    				bn[ynS+nS][xnS].setI2(input[j+1]);
	    	    			bn[ynS+nS][xnS].setExchange();
    		    		}
	    		    	else
		    		    {
    		    			sV = input[j-1];
	    		    		bn[ynS+nS][xnS].setI1(input[j-1]);
		    		    	bn[ynS+nS][xnS].setI2(input[j]);
    			    	}
    				    return nS;
        			}
    	    	}
            }
        }
        else
        {
    		for( j = 0; j < ylS*2; j++ )
	    	{
    			if( output[j] == input[inS*2] )
	    		{
		    		nS = j/2;
			    	o1[nS] = output[j];
				    if( j%2 == 0 )
    				{
	    				sV = output[j+1];
		    			bn[ynS+nS][xnS + xlS - 1].setI1(output[j]);
			    		bn[ynS+nS][xnS + xlS - 1].setI2(output[j+1]);
				    }
    				else
	    			{
		    			sV = output[j-1];
    					bn[ynS+nS][xnS + xlS - 1].setI2(output[j-1]);
	    				bn[ynS+nS][xnS + xlS - 1].setI1(output[j]);
		    			bn[ynS+nS][xnS + xlS - 1].setExchange();
			    	}
    				bn[ynS+nS][xnS + xlS - 1].setVisited();
	    			break;
		    	}
            }

    		if( ylS*2 == j ) return -1;

	    	// Loop backward
		    o2[nS] = sV;
    		for( j = 0; j < ylS*2; j++ )
	    	{
		    	if( input[j] == sV )
			    {
    				nS = j/2;
	    			i2[nS] = sV;
		    		if( j%2 == 0 )
			    	{
				    	sV = input[j+1];
					    bn[ynS+nS][xnS].setI1(input[j]);
    					bn[ynS+nS][xnS].setI2(input[j+1]);
	    				bn[ynS+nS][xnS].setExchange();
		    		}
			    	else
				    {
    					sV = input[j-1];
	    				bn[ynS+nS][xnS].setI1(input[j-1]);
		    			bn[ynS+nS][xnS].setI2(input[j]);
			    	}
//    				bn[ynS+nS][xnS].setVisited();
				    return nS;
    			}
	    	}
        }
        return -1;
    }

    /**
     * Gets switch number on y-axis by x-axis and one of values of the switch
     *
     * @param l - switch number on x-axis
     * @param v - value of the switch
     * @return
     */
	public int getSwitchNumber( int l, int v )
	{
		for( int i = 0; i < gyS; i++ )
		{
			if( bn[i][l].getI1() == v || bn[i][l].getI2() == v ) return i;
		}
		return -1;
	}


    /**
     * Executes Benes Network algorithm for sub network for looping
     *
     * @param input     - array of input values for sub network
     * @param output    - array of output values for sub network
     * @param xnS       - x-coordinate of the switch in the Network
     * @param ynS       - y-coordinate of the switch in the Network
     * @param xlS       - number of switches in subnetwork for x-axis
     * @param ylS       - number of switches in subnetwork for y-axis
     * @param it        - number of iterations to do
     */
    public void execute_loop( int[] input, int[] output,
		int xnS, int ynS, int xlS, int ylS, int it )
    {
		this.iType = iType;

        if( nit == 0 ) return;

        // Sets parameters for iterations
        if( it > 0 )    { nit = it; it = 0; }

		// Recursion condition
		if( xlS == 1 )
		{
	        for( int i = 0; i < ylS; i++ )
			{
				bn[ynS+i][xnS].setVisited();
				bn[ynS+i][xnS].setI1( input[i*2] );
				bn[ynS+i][xnS].setI2( input[i*2+1] );
				if( input[i*2] != output[i*2] )
					bn[ynS+i][xnS].setExchange();

                nit--;
                if( nit == 0 ) return;
			}
			return;
		}

        // Initialization
		int[] i1 = new int[ ylS ];
		int[] i2 = new int[ ylS ];
		int[] o1 = new int[ ylS ];
		int[] o2 = new int[ ylS ];
		int nS;

        for( int i = 0; i < ylS; i++ )
        {
            i1[i] = i2[i] = o1[i] = o2[i] = -1;
        }

        // Processing through the input switches
        for( int i = 0; i < ylS; i++ )
		{
			if( bn[ynS+i][xnS].isVisited() ) continue;

			nS = i;

			do
			{
                // End of the loop iteration condition
//                if( iType == 1 )
//                    if( nit != -1 )
//                    {
//                        setSwitches( i1, o1, xnS+1, ynS, xlS-2, ylS/2 );
//                        setSwitches( i2, o2, xnS+1, ynS+ylS/2, xlS-2, ylS/2 );
//                        if( nit == 0 ) return;
//                        nit--;
//                    }
                //--

                // Assign values to input and output arrays
                if( bn[ynS+nS][xnS].isVisited() )
                {
    				bn[ynS+nS][xnS].setI2( input[nS*2+1] );
                    if(bn[ynS+nS][xnS].isExchanged())
        				i1[nS] = bn[ynS+nS][xnS].getI2();
                    else
        				i2[nS] = bn[ynS+nS][xnS].getI2();
                }
                else
                {
    				bn[ynS+nS][xnS].setI1( input[nS*2] );
                    if(bn[ynS+nS][xnS].isExchanged())
        				i2[nS] = bn[ynS+nS][xnS].getI1();
                    else
        				i1[nS] = bn[ynS+nS][xnS].getI1();
                }

				bn[ynS+nS][xnS].setVisited();

                setSwitches( i1, o1, xnS+1, ynS, xlS-2, ylS/2 );
                setSwitches( i2, o2, xnS+1, ynS+ylS/2, xlS-2, ylS/2 );
                nit--;
                if( nit == 0 ) return;

                // Execute one loop through the algorithm
				nS = loop_loop( input, output, xnS, ynS, xlS, ylS, i1, i2, o1, o2, nS );

                setSwitches( i1, o1, xnS+1, ynS, xlS-2, ylS/2 );
                setSwitches( i2, o2, xnS+1, ynS+ylS/2, xlS-2, ylS/2 );
                if( nit == 0 ) return;


				if( nS == -1 )
				{
					break;
				}
                if(!bn[ynS+nS][xnS].isVisited())
                {
                    bn[ynS+nS][xnS].setVisited();
    				bn[ynS+nS][xnS].setI1( input[nS*2] );
                    if(bn[ynS+nS][xnS].isExchanged())
        				i2[nS] = bn[ynS+nS][xnS].getI1();
                    else
        				i1[nS] = bn[ynS+nS][xnS].getI1();

                    setSwitches( i1, o1, xnS+1, ynS, xlS-2, ylS/2 );
                    setSwitches( i2, o2, xnS+1, ynS+ylS/2, xlS-2, ylS/2 );
                    nit--;
                    if( nit == 0 ) return;

                    continue;
                }
                else    break;
			}
			while( true /*!bn[ynS+nS][xnS].isVisited()*/ );
		}

//        if( iType != 1 )
        {
            setSwitches( i1, o1, xnS+1, ynS, xlS-2, ylS/2 );
            setSwitches( i2, o2, xnS+1, ynS+ylS/2, xlS-2, ylS/2 );
        }

        // Execute algorithm for subnetworks
		execute_loop( i1, o1, xnS+1, ynS, xlS-2, ylS/2, /*it-1*/ 0 );
		execute_loop( i2, o2, xnS+1, ynS+ylS/2, xlS-2, ylS/2, /*it-1*/ 0 );
    }

        /**
     * Single loop algorithm for looping
     *
     * @param input     - input values
     * @param output    - output values
     * @param xnS       - x-coordinate of the switch in the Network
     * @param ynS       - y-coordinate of the switch in the Network
     * @param xlS       - number of switches in subnetwork for x-axis
     * @param ylS       - number of switches in subnetwork for y-axis
     * @param i1        - input values for upper sub network
     * @param i2        - input values for lower sub network
     * @param o1        - output values for upper sub network
     * @param o2        - output values for lower sub network
     * @param inS
     * @return  switch number for the following loop iteration
     */
	private int loop_loop(int[] input, int[] output,
		int xnS, int ynS, int xlS, int ylS,
		int[] i1, int[] i2, int[] o1, int[] o2, int inS)
	{
		int sV = -1; // switch value
		int nS = -1; // switch number

		// Loop forward
		int j;
        if( bn[ynS+inS][xnS].isVisited() )
        {
            if( bn[ynS+inS][xnS].isExchanged() )
            {
                // Visited and Exchanged
        		for( j = 0; j < ylS*2; j++ )
	        	{
    	    		if( output[j] == input[inS*2+1] )
	    	    	{
    		    		nS = j/2;
	    		    	o1[nS] = output[j];
		    		    if( j%2 == 0 )
    		    		{
	    		    		sV = output[j+1];
		    		    	bn[ynS+nS][xnS + xlS - 1].setI1(output[j]);
			    		    bn[ynS+nS][xnS + xlS - 1].setI2(output[j+1]);
    				    }
        				else
	        			{
		        			sV = output[j-1];
    			    		bn[ynS+nS][xnS + xlS - 1].setI2(output[j-1]);
	    			    	bn[ynS+nS][xnS + xlS - 1].setI1(output[j]);
		    			    bn[ynS+nS][xnS + xlS - 1].setExchange();
    			    	}
        				bn[ynS+nS][xnS + xlS - 1].setVisited();

                        nit--;
                        if( nit == 0 ) return -1;

	        			break;
                    }
                }
        		if( ylS*2 == j ) return -1;

    	    	// Loop backward
	    	    o2[nS] = sV;
    	    	for( j = 0; j < ylS*2; j++ )
	    	    {
    		    	if( input[j] == sV )
	    		    {
    	    			nS = j/2;
	    	    		i2[nS] = sV;
		    	    	if( j%2 == 0 )
			    	    {
    				    	sV = input[j+1];
	    				    bn[ynS+nS][xnS].setI1(input[j]);
    	    				bn[ynS+nS][xnS].setI2(input[j+1]);
	    	    			bn[ynS+nS][xnS].setExchange();
		    	    	}
    			    	else
	    			    {
    	    				sV = input[j-1];
	    	    			bn[ynS+nS][xnS].setI1(input[j-1]);
		    	    		bn[ynS+nS][xnS].setI2(input[j]);
			    	    }
                        nit--;
                        if( nit == 0 ) return -1;
    				    return nS;
        			}
	        	}
            }
            else
            {
        		for( j = 0; j < ylS*2; j++ )
    	    	{
    	    		if( output[j] == input[inS*2] )
	    	    	{
		    	    	nS = j/2;
			    	    o1[nS] = output[j];
    				    if( j%2 == 0 )
        				{
	        				sV = output[j+1];
		        			bn[ynS+nS][xnS + xlS - 1].setI1(output[j]);
			        		bn[ynS+nS][xnS + xlS - 1].setI2(output[j+1]);
				        }
        				else
	        			{
		        			sV = output[j-1];
    		    			bn[ynS+nS][xnS + xlS - 1].setI2(output[j-1]);
	    		    		bn[ynS+nS][xnS + xlS - 1].setI1(output[j]);
		    		    	bn[ynS+nS][xnS + xlS - 1].setExchange();
    			    	}
        				bn[ynS+nS][xnS + xlS - 1].setVisited();

                        nit--;
                        if( nit == 0 ) return -1;

	        			break;
		        	}
                }

        		if( ylS*2 == j ) return -1;

    	    	// Loop backward
	    	    o2[nS] = sV;

                nit--;
                if( nit == 0 ) return -1;

    	    	for( j = 0; j < ylS*2; j++ )
	    	    {
    		    	if( input[j] == sV )
	    		    {
    	    			nS = j/2;
	    	    		i2[nS] = sV;
		    	    	if( j%2 == 0 )
			    	    {
    				    	sV = input[j+1];
	    				    bn[ynS+nS][xnS].setI1(input[j]);
    	    				bn[ynS+nS][xnS].setI2(input[j+1]);
	    	    			bn[ynS+nS][xnS].setExchange();
    		    		}
	    		    	else
		    		    {
    		    			sV = input[j-1];
	    		    		bn[ynS+nS][xnS].setI1(input[j-1]);
		    		    	bn[ynS+nS][xnS].setI2(input[j]);
    			    	}

                        nit--;
                        if( nit == 0 ) return -1;

    				    return nS;
        			}
    	    	}
            }
        }
        else
        {
    		for( j = 0; j < ylS*2; j++ )
	    	{
    			if( output[j] == input[inS*2] )
	    		{
		    		nS = j/2;
			    	o1[nS] = output[j];
				    if( j%2 == 0 )
    				{
	    				sV = output[j+1];
		    			bn[ynS+nS][xnS + xlS - 1].setI1(output[j]);
			    		bn[ynS+nS][xnS + xlS - 1].setI2(output[j+1]);
				    }
    				else
	    			{
		    			sV = output[j-1];
    					bn[ynS+nS][xnS + xlS - 1].setI2(output[j-1]);
	    				bn[ynS+nS][xnS + xlS - 1].setI1(output[j]);
		    			bn[ynS+nS][xnS + xlS - 1].setExchange();
			    	}
    				bn[ynS+nS][xnS + xlS - 1].setVisited();

                    nit--;
                    if( nit == 0 ) return -1;

	    			break;
		    	}
            }

    		if( ylS*2 == j ) return -1;

	    	// Loop backward
		    o2[nS] = sV;
    		for( j = 0; j < ylS*2; j++ )
	    	{
		    	if( input[j] == sV )
			    {
    				nS = j/2;
	    			i2[nS] = sV;
		    		if( j%2 == 0 )
			    	{
				    	sV = input[j+1];
					    bn[ynS+nS][xnS].setI1(input[j]);
    					bn[ynS+nS][xnS].setI2(input[j+1]);
	    				bn[ynS+nS][xnS].setExchange();
		    		}
			    	else
				    {
    					sV = input[j-1];
	    				bn[ynS+nS][xnS].setI1(input[j-1]);
		    			bn[ynS+nS][xnS].setI2(input[j]);
			    	}
//    				bn[ynS+nS][xnS].setVisited();
                    nit--;
                    if( nit == 0 ) return -1;

				    return nS;
    			}
	    	}
        }
        return -1;
    }


}
